#ifndef KYSEC_LOG_H
#define KYSEC_LOG_H

//size for the function back strace
#define BT_BUF_SIZE 32

enum kysec_log_type {
	KYSECLOG_LIB_KYSEC = 0,		/* libkysec */
	KYSECLOG_LIB_EXTEND,		/* libkysec_extend */
	KYSECLOG_LIB_KYSECWHLIST,	/* libkysecwhlist */
	KYSECLOG_LIB_CORE,		/* libkysec_core */
	KYSECLOG_LIB_ADV,		/* libkysec_adv */
	KYSECLOG_CMD_SETSTATUS = 5,	/* setstatus */
	KYSECLOG_CMD_GETSTATUS,		/* getstatus */
	KYSECLOG_CMD_KYSEC,		/* kysec */
	KYSECLOG_CMD_SYNC,		/* kysec-sync */
	KYSECLOG_CMD_SYNC_DAEMON,	/* kysec-sync-daemon */
	KYSECLOG_CMD_DAEMON,		/* kysec-daemon */
	KYSECLOG_CMD_AUTH,		/* kysec_auth */
	KYSECLOG_CMD_KMOD = 12,		/* kysec_kmod */
	KYSECLOG_CMD_KSC_DEFENDER = 13,	/* ksc_defender	*/
	KYSECLOG_CMD_KSC_DEFENDER_DAEMON = 14,	/* ksc_defender_daemon	*/
	KYSECLOG_LOG,
	KYSECLOG_LIB_LOG,
	KYSECTRACELOG,          /* KYSEC TRACE LOG SP4 ADDED*/
	KYSECLOG_MAX			/* kysec log max type, limited value */
};

enum { DEBUG = 0, INFO = 1, WARN = 2, ERROR = 3, FATAL = 4, TRACE=5};

#ifdef __cplusplus
extern "C" {
#endif

/* kysec日志记录接口
 * @type: 取值kysec_log_type枚举
 * @op: 指添加、删除、更新白名单等具体操作，该字符串长度不能超过100
 * @obj: 指当前的操作对象，该字符串长度不能超过PATH_MAX
 * @format: 指具体的失败信息，或想要记录的任何信息均可，该字符串长度不能超过1024
 */
void kysec_log(int type, char *op, char *obj, char *format, ...);

/* kysec日志记录接口，sp4新增，记录更多日志字段
 * @type: 取值kysec_log_type枚举
 * @op: 指添加、删除、更新白名单等具体操作，该字符串长度不能超过100
 * @obj: 指当前的操作对象，该字符串长度不能超过PATH_MAX
 * @filename: 指记录日志的代码源文件名称，字符串长度不能超过FILENAME_MAX
 * @line: 指记录日志的代码原文件的行号
 * @format: 指具体的失败信息，或想要记录的任何信息均可，该字符串长度不能超过1024
 */
void kysec_log_v2(int level, int type, char *op, char *obj,
                        char* filename, int line, char *format, ...);
/*
 * 获取函数栈调用信息
 * @filename: 进程名
 * @line: 行号
 * @function: 函数名
 */
void kysec_log_trace(char* filename, int line, char* function);

/*
 * 记录日志的宏定义：
 * 方式一：KYSEC_LOG(LEVEL)(level, type, op, obj, format, args...)
 * 方式二：KYSEC_LOG_INFO(type, op, obj, format, args...)
 */
#define KYSEC_LOG(LEVEL) KYSEC_LOG_##LEVEL
#define KYSEC_LOG_V2(level, type, op, obj, format, args...) do{                 \
    kysec_log_v2(level, type, op, obj, (char*)__FILE__, (int)__LINE__, format, ##args);     \
    }while(0);

#define KYSEC_LOG_DEBUG(type, op, obj, format, args...) KYSEC_LOG_V2(DEBUG, type, op, obj, format, ##args)
#define KYSEC_LOG_INFO(type, op, obj, format, args...) KYSEC_LOG_V2(INFO, type, op, obj, format, ##args)
#define KYSEC_LOG_WARN(type, op, obj, format, args...) KYSEC_LOG_V2(WARN, type, op, obj, format, ##args)
#define KYSEC_LOG_ERROR(type, op, obj, format, args...) KYSEC_LOG_V2(ERROR, type, op, obj, format, ##args)
#define KYSEC_LOG_FATAL(type, op, obj, format, args...) do{     \
    kysec_log_v2(FATAL, type, op, obj, (char*)__FILE__, (int)__LINE__, format, ##args);     \
    kysec_log_trace((char*)__FILE__, (int)__LINE__, (char*)__FUNCTION__);                   \
    }while(0);
#define KYSEC_LOG_TRACE() kysec_log_trace((char*)__FILE__, (int)__LINE__, (char*)__FUNCTION__)
#ifdef __cplusplus
}
#endif

#endif /* KYSEC_LOG_H */
